﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    internal class SetPlanesGeneric<T>
        where T: class 
    {
        //список объектов, которые храним
        private readonly List<T> _places;

        //количество объектов в массиве
        public int Count => _places.Count;

        //ограничение по кол-ву объектов
        private readonly int _maxCount;

        //конструктор
        public SetPlanesGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }

        //добавление объекта в набор
        public int Insert(T plane)
        {
            //вставка в начало набора
            //проверка на максим. кол-во////////////////////////////////////////////////////////////

            return Insert(plane, 0);
        }

        //добавление объекта в набор на конкретную позицию
        public int Insert(T plane, int position)
        {
            //  проверка позиции
            if (position >= _places.Count || position < 0)
            {
                return -1;
            }

            //проверка, что элемент массива по этой позиции пустой, если нет, то
            if (_places[position] == null)
            {
                _places[position] = plane;
                return position;
            }

            //проверка, что после вставляемого элемента в массиве есть пустой элемент
            int findEmptyPos = -1;

            for (int i = position + 1; i < Count; i++)
            {
                if (_places[i] == null)
                {
                    findEmptyPos = i;
                    break;
                }
            }

            if (findEmptyPos < 0)
            {
                return -1;
            }

            //сдвиг всех объектов, находящихся справа от позиции до первого пустого элемента
            for (int i = findEmptyPos; i > position; i--)
            {
                _places[i] = _places[i - 1];
            }

            // вставка по позиции
            _places[position] = plane;

            return position;
        }

        //удаление объекта из набора с конкретной позиции
        public T Remove(int position)
        {
            // проверка позиции
            if (position >= _places.Count || position < 0)
            {
                return null;
            }

            // удаление объекта из массива, присовив элементу массива значение null
            T temp = _places[position];
            _places[position] = null;

            return temp;
        }

        //получение объекта из набора по позиции
        public T this[int position]
        {
            get
            {
                if (position >= _places.Count || position < 0)
                {
                    return null;
                }
                else if (_places[position] == null)
                {
                    return null;
                }

                return _places[position];
            }
            set
            {
                //TODO проверка позиции
                //TODO вставка в список по позиции
            }
        }

        //проход по набору до первого пустого
        public IEnumerable<T> GetAirbus()
        {
            foreach(var plane in _places)
            {
                if(plane != null)
                {
                    yield return plane;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
